/*
 * Copyright (c) 2003, Greg McIntyre
 * All rights reserved.
 */

/**
 * \mainpage Field of View Library
 * 
 * \section about About
 * 
 * This is a C library which implements a course-grained lighting
 * algorithm suitable for tile-based games such as roguelikes.
 * 
 * \section copyright Copyright
 * 
 * \verbinclude COPYING
 * 
 * \section thanks Thanks
 * 
 * Thanks to Bj&ouml;rn Bergstr&ouml;m
 * <bjorn.bergstrom@hyperisland.se> for the algorithm.
 * 
 */

/**
 * \file fov.h
 * Field-of-view algorithm for dynamically casting light/shadow on a
 * low resolution 2D raster.
 */
#ifndef FOV_HEADER
#define FOV_HEADER

#ifdef __cplusplus
extern "C" {
#endif

/** Eight-way directions. */
typedef enum {
	east = 0,
	northeast,
	north,
	northwest,
	west,
	southwest,
	south,
	southeast
} fov_direction_type;

/** The opposite direction to that given. */
#define fov_direction_opposite(direction) ((fov_direction_type)(((direction)+4)&0x7))

/**
 * Limit the FOV to a circle with radius R by calculating on-the-fly.
 */
void fov_set_limit_circular();

/**
 * \b (default) Limit the FOV to a circle with radius R by
 * precalculating, which consumes more memory at the rate of 4*(R+2)
 * bytes per R used in calls to fov_circle. Each radius is only
 * calculated once so that it can be used again. Use fov_free() to
 * free this precalculated data's memory.
 */
void fov_set_limit_circular_precalculated();

/**
 * Limit the FOV to an octogon with maximum radius R.
 */
void fov_set_limit_octagonal();

/**
 * Limit the FOV to an R*R square.
 */
void fov_set_limit_square();

/**
 * \b (default) Sources will not peek around corners.
\verbatim
  ......
  .....
  ....
  ..@#
  ...#
\endverbatim
 */
void fov_set_nopeek();

/**
 * <em>NOT YET IMPLEMENTED</em>.
 * Sources will peek around corners.
\verbatim
  ........
  ........
  ........
  ..@#    
  ...#    
\endverbatim
 */
void fov_set_peek();

/**
 * \b (default) Call the apply callback on opaque tiles.
 */
void fov_set_opaque_apply();

/**
 * Do not call the apply callback on opaque tiles.
 */
void fov_set_opaque_noapply();

/**
 * Free all memory allocated on the heap through any means.
 * libfov does not necessarily deallocate memory used for
 * precalculation as it does not know when this will be used again. It
 * is therefore up to you to tell libfov to release its memory for any
 * reason.
 */
void fov_free();


/**
 * Calculate a full circle field of view from a source at (x,y).
 *
 * \param map Pointer to map data structure to be passed to callbacks.
 * \param src Pointer to data structure holding source of light.
 * \param x x-axis coordinate from which to start.
 * \param y y-axis coordinate from which to start.
 * \param radius Euclidean distance from (x,y) after which to stop.
 * \param opaque Predicate used to deterimine if light will \a not pass through a particular coordinate (if it is opaque).
 * \param apply Callback called once per coordinate in the field of view of (x,y).
 */
void fov_circle(void *map, void *src, int x, int y, unsigned radius,
	int (*opaque)(void *map, int x, int y),
	void (*apply)(void *map, int x, int y, int dx, int dy, void *src)
);

/**
 * <em>NOT YET IMPLEMENTED</em>.
 * Calculate a field of view from source at (x,y), pointing
 * in the given direction and with the given angle. The larger
 * the angle, the wider, "less focused" the beam. Each side of the
 * line pointing in the direction from the source will be half the
 * angle given such that the angle specified will be represented on
 * the raster.
 *
 * \param map Pointer to map data structure to be passed to callbacks.
 * \param src Pointer to data structure holding source of light.
 * \param x x-axis coordinate from which to start.
 * \param y y-axis coordinate from which to start.
 * \param radius Euclidean distance from (x,y) after which to stop.
 * \param opaque Predicate used to deterimine if light will \a not pass through a particular coordinate (if it is opaque).
 * \param apply Callback called once per coordinate in the field of view of (x,y).
 * \param direction One of eight directions the beam of light can point.
 * \param angle The angle at the base of the beam of light, in degrees.
 */
void fov_beam(void *map, void *src, int x, int y, unsigned radius,
	int (*opaque)(void *map, int x, int y),
	void (*apply)(void *map, int x, int y, int dx, int dy, void *src),
	fov_direction_type direction, float angle
);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif
